package main

import (
	"testing"
	"gitlab.smartblocklab.com/smartcontracts/systempreferenceCC/mngr"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/model/preferenceCC"
	"github.com/stretchr/testify/assert"
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
	mock2 "github.com/stretchr/testify/mock"
	"errors"
)

func TestManager_create(t *testing.T) {
	manager := mngr.GetSystemPreferenceManager(chaincode.ManagerImplMock{})

	assert.NotNil(t, manager)
}

func createPreference() *preferenceCC.Preference {
	preference := &preferenceCC.Preference{}
	preference.Key = "key"
	fields := chaincode.Fields{}
	fields["key1"] = "val1"
	fields["key2"] = "val2"
	preference.Fields = fields
	preference.Validate()
	return preference
}

func TestManager_addPreference_RETURN_nil(t *testing.T) {
	mock := chaincode.ManagerImplMock{}
	mock.On("AddObject", mock2.Anything).Return(nil, nil)

	manager := mngr.GetSystemPreferenceManager(mock)

	preference, err := manager.AddPreference(createPreference())

	assert.NoError(t, err)
	assert.Nil(t, preference)
}

func TestManager_addPreference_RETURN_preference(t *testing.T) {
	preference := createPreference()

	mock := chaincode.ManagerImplMock{}
	mock.On("AddObject", mock2.Anything).Return(preference, nil)

	manager := mngr.GetSystemPreferenceManager(mock)

	preference, err := manager.AddPreference(preference)

	assert.NoError(t, err)
	assert.NotNil(t, preference)
}

func TestManager_addPreference_RETURN_err(t *testing.T) {
	preference := createPreference()

	mock := chaincode.ManagerImplMock{}
	mock.On("AddObject", mock2.Anything).Return(nil, errors.New("some problem"))

	manager := mngr.GetSystemPreferenceManager(mock)

	preference, err := manager.AddPreference(preference)

	assert.Error(t, err)
	assert.Nil(t, preference)
}

func TestManager_getPreference_RETURN_preference(t *testing.T) {
	mock := chaincode.ManagerImplMock{}
	found := createPreference()
	mock.On("GetObject", mock2.Anything, mock2.Anything).Return(found, nil)

	manager := mngr.GetSystemPreferenceManager(mock)

	preference, err := manager.GetPreference(&requests.Id{Id: "key"})

	assert.NoError(t, err)
	assert.NotNil(t, preference)
}

func TestManager_getPreference_RETURN_err(t *testing.T) {
	mock := chaincode.ManagerImplMock{}
	errMsg := "some problem"
	mock.On("GetObject", mock2.Anything, mock2.Anything).Return(nil, errors.New(errMsg))

	manager := mngr.GetSystemPreferenceManager(mock)

	preference, err := manager.GetPreference(&requests.Id{Id: "key"})

	assert.Error(t, err)
	assert.Nil(t, preference)
}

func TestManager_updatePreference_RETURN_error(t *testing.T) {
	mock := chaincode.ManagerImplMock{}
	mock.On("UpdateObject", mock2.Anything).Return(nil, errors.New("some problem"))

	manager := mngr.GetSystemPreferenceManager(mock)

	preference, err := manager.UpdatePreference(createPreference())

	assert.Error(t, err)
	assert.Nil(t, preference)
}

func TestManager_updatePreference_RETURN_preference(t *testing.T) {
	mock := chaincode.ManagerImplMock{}
	newPreference := createPreference()
	mock.On("UpdateObject", mock2.Anything).Return(newPreference, errors.New("some problem"))

	manager := mngr.GetSystemPreferenceManager(mock)

	preference, err := manager.UpdatePreference(newPreference)

	assert.Error(t, err)
	assert.Nil(t, preference)
}
