package actions

import (
	log "github.com/sirupsen/logrus"
	"gitlab.smartblocklab.com/smartcontracts/common/logging"
	"strings"
	"errors"
)

type ActionIDsObject struct {
}

type ParameterType int

type Roles string

const (
	UserRole   Roles = "USER"
	SystemRole Roles = "SYSTEM_USER"
	RootRole   Roles = "ROOT_USER"
)

type SecurityPreference struct {
	ChaincodeName string
	FunctionName  string
	ParameterType ParameterType
	Roles         []Roles
}

var (
	logger = log.WithFields(log.Fields{})
)

func (securityPreference SecurityPreference) GetActionID(channelName string) int32 {
	actionID := channelName + securityPreference.ChaincodeName + securityPreference.FunctionName
	logger.Infof("ActionId source: '%s'", actionID)
	var hash int32
	hash = Hash(actionID)
	logger.Infof("ActionId hash: '%d'", hash)
	return hash
}

func Hash(s string) int32 {
	var h int32
	h = 0
	var ln int32
	var li int32
	ln = int32(len(s))
	li = ln - 1 // li means last index
	var i int32
	for i = 0; i < ln; i++ {
		h += int32(s[i]) * Pow(31, li-i)
	}
	return h
}

func Pow(x int32, y int32) int32 {
	p := x
	if y == 0 {
		return 1
	}
	if y == 1 {
		return x
	}
	// if y < 0, panic?
	var i int32
	for i = 1; i < y; i++ {
		p *= x
	}
	return p
}

const UserParameter ParameterType = 1
const ServiceParameter ParameterType = 1 << 2
const ComplexParameter ParameterType = 1 << 3
const UnsecuredParameter ParameterType = 1 << 4

func FindSecurityPreferenceByFunctionName(chaincodeName string, functionName string) (SecurityPreference, error) {
	logging.Get().Infof("chaincodeName: %s", chaincodeName)
	logging.Get().Infof("functionName: %s", functionName)
	chaincodeNameLower := firstLetterToLowerCase(chaincodeName)
	functionNameLower := firstLetterToLowerCase(functionName)

	for _, v := range SecurityPreferenceArray {
		if v.ChaincodeName == chaincodeNameLower && v.FunctionName == functionNameLower {
			logging.Get().Infof("found SecurityPreference: ChaincodeName %s, FunctionName %s", v.ChaincodeName, v.FunctionName)
			return v, nil
		}
	}
	return SecurityPreference{}, errors.New("SECURITY_FOR_FUNCTION_NOT_FOUND")
}

func firstLetterToLowerCase(str string) string {
	firstLetter := strings.ToLower(str[0:1])
	replaced := firstLetter + str[1:]

	return replaced
}
